@@ -37,6 +37,7 @@ type ColumnBuilder struct {
 	attr   string             // extra attributes.
 	modify bool               // modify existing.
 	fk     *ForeignKeyBuilder // foreign-key constraint.
+	check  func(*Builder)     // column checks.
 }
 
 // Column returns a new ColumnBuilder with the given name.
@@ -66,12 +67,18 @@ func (c *ColumnBuilder) Constraint(fk *ForeignKeyBuilder) *ColumnBuilder {
 	return c
 }
 
+// Check adds a CHECK clause to the ADD COLUMN statement.
+func (c *ColumnBuilder) Check(check func(*Builder)) *ColumnBuilder {
+	c.check = check
+	return c
+}
+
 // Query returns query representation of a Column.
 func (c *ColumnBuilder) Query() (string, []interface{}) {
 	c.Ident(c.name)
 	if c.typ != "" {
 		if c.postgres() && c.modify {
-			c.Pad().WriteString("TYPE")
+			c.WriteString(" TYPE")
 		}
 		c.Pad().WriteString(c.typ)
 	}
@@ -85,6 +92,10 @@ func (c *ColumnBuilder) Query() (string, []interface{}) {
 			c.Pad().WriteString(action)
 		}
 	}
+	if c.check != nil {
+		c.WriteString(" CHECK ")
+		c.Nested(c.check)
+	}
 	return c.String(), c.args
 }
 
@@ -605,6 +616,7 @@ func (d *DropIndexBuilder) Query() (string, []interface{}) {
 type InsertBuilder struct {
 	Builder
 	table     string
+	schema    string
 	columns   []string
 	defaults  string
 	returning []string
@@ -621,6 +633,12 @@ type InsertBuilder struct {
 // Note: Insert inserts all values in one batch.
 func Insert(table string) *InsertBuilder { return &InsertBuilder{table: table} }
 
+// Schema sets the database name for the insert table.
+func (i *InsertBuilder) Schema(name string) *InsertBuilder {
+	i.schema = name
+	return i
+}
+
 // Set is a syntactic sugar API for inserting only one row.
 func (i *InsertBuilder) Set(column string, v interface{}) *InsertBuilder {
 	i.columns = append(i.columns, column)
@@ -664,6 +682,7 @@ func (i *InsertBuilder) Returning(columns ...string) *InsertBuilder {
 // Query returns query representation of an `INSERT INTO` statement.
 func (i *InsertBuilder) Query() (string, []interface{}) {
 	i.WriteString("INSERT INTO ")
+	i.writeSchema(i.schema)
 	i.Ident(i.table).Pad()
 	if i.defaults != "" && len(i.columns) == 0 {
 		i.WriteString(i.defaults)
@@ -692,6 +711,7 @@ func (i *InsertBuilder) Query() (string, []interface{}) {
 type UpdateBuilder struct {
 	Builder
 	table   string
+	schema  string
 	where   *Predicate
 	nulls   []string
 	columns []string
@@ -704,6 +724,12 @@ type UpdateBuilder struct {
 //
 func Update(table string) *UpdateBuilder { return &UpdateBuilder{table: table} }
 
+// Schema sets the database name for the updated table.
+func (u *UpdateBuilder) Schema(name string) *UpdateBuilder {
+	u.schema = name
+	return u
+}
+
 // Set sets a column and a its value.
 func (u *UpdateBuilder) Set(column string, v interface{}) *UpdateBuilder {
 	u.columns = append(u.columns, column)
@@ -758,7 +784,8 @@ func (u *UpdateBuilder) Empty() bool {
 // Query returns query representation of an `UPDATE` statement.
 func (u *UpdateBuilder) Query() (string, []interface{}) {
 	u.WriteString("UPDATE ")
-	u.Ident(u.table).Pad().WriteString("SET ")
+	u.writeSchema(u.schema)
+	u.Ident(u.table).WriteString(" SET ")
 	for i, c := range u.nulls {
 		if i > 0 {
 			u.Comma()
@@ -790,8 +817,9 @@ func (u *UpdateBuilder) Query() (string, []interface{}) {
 // DeleteBuilder is a builder for `DELETE` statement.
 type DeleteBuilder struct {
 	Builder
-	table string
-	where *Predicate
+	table  string
+	schema string
+	where  *Predicate
 }
 
 // Delete creates a builder for the `DELETE` statement.
@@ -810,6 +838,12 @@ type DeleteBuilder struct {
 //
 func Delete(table string) *DeleteBuilder { return &DeleteBuilder{table: table} }
 
+// Schema sets the database name for the table whose row will be deleted.
+func (d *DeleteBuilder) Schema(name string) *DeleteBuilder {
+	d.schema = name
+	return d
+}
+
 // Where appends a where predicate to the `DELETE` statement.
 func (d *DeleteBuilder) Where(p *Predicate) *DeleteBuilder {
 	if d.where != nil {
@@ -832,6 +866,7 @@ func (d *DeleteBuilder) FromSelect(s *Selector) *DeleteBuilder {
 // Query returns query representation of a `DELETE` statement.
 func (d *DeleteBuilder) Query() (string, []interface{}) {
 	d.WriteString("DELETE FROM ")
+	d.writeSchema(d.schema)
 	d.Ident(d.table)
 	if d.where != nil {
 		d.WriteString(" WHERE ")
@@ -1208,6 +1243,10 @@ func (p *Predicate) Append(f func(*Builder)) *Predicate {
 
 // Query returns query representation of a predicate.
 func (p *Predicate) Query() (string, []interface{}) {
+	if p.Len() > 0 || len(p.args) > 0 {
+		p.Reset()
+		p.args = nil
+	}
 	for _, f := range p.fns {
 		f(&p.Builder)
 	}
@@ -1416,8 +1455,8 @@ func (s *SelectTable) C(column string) string {
 		name = s.as
 	}
 	b := &Builder{dialect: s.dialect}
-	if s.schema != "" && s.as == "" {
-		b.Ident(s.schema).WriteByte('.')
+	if s.as == "" {
+		b.writeSchema(s.schema)
 	}
 	b.Ident(name).WriteByte('.').Ident(column)
 	return b.String()
@@ -1446,9 +1485,7 @@ func (s *SelectTable) ref() string {
 		return s.name
 	}
 	b := &Builder{dialect: s.dialect}
-	if s.schema != "" {
-		b.Ident(s.schema).WriteByte('.')
-	}
+	b.writeSchema(s.schema)
 	b.Ident(s.name)
 	if s.as != "" {
 		b.WriteString(" AS ")
@@ -2012,6 +2049,12 @@ func (b *Builder) AddError(err error) *Builder {
 	return b
 }
 
+func (b *Builder) writeSchema(schema string) {
+	if schema != "" && b.dialect != dialect.SQLite {
+		b.Ident(schema).WriteByte('.')
+	}
+}
+
 // Err returns a concatenated error of all errors encountered during
 // the query-building, or were added manually by calling AddError.
 func (b *Builder) Err() error {
