@@ -78,11 +78,14 @@ func TestBuilder(t *testing.T) {
 				Columns(
 					Column("id").Type("int").Attr("auto_increment"),
 					Column("card_id").Type("int"),
+					Column("doc").Type("longtext").Check(func(b *Builder) {
+						b.WriteString("JSON_VALID(").Ident("doc").WriteByte(')')
+					}),
 				).
 				PrimaryKey("id", "name").
 				ForeignKeys(ForeignKey().Columns("card_id").
 					Reference(Reference().Table("cards").Columns("id")).OnDelete("SET NULL")),
-			wantQuery: "CREATE TABLE IF NOT EXISTS `users`(`id` int auto_increment, `card_id` int, PRIMARY KEY(`id`, `name`), FOREIGN KEY(`card_id`) REFERENCES `cards`(`id`) ON DELETE SET NULL)",
+			wantQuery: "CREATE TABLE IF NOT EXISTS `users`(`id` int auto_increment, `card_id` int, `doc` longtext CHECK (JSON_VALID(`doc`)), PRIMARY KEY(`id`, `name`), FOREIGN KEY(`card_id`) REFERENCES `cards`(`id`) ON DELETE SET NULL)",
 		},
 		{
 			input: Dialect(dialect.Postgres).CreateTable("users").
@@ -220,11 +223,26 @@ func TestBuilder(t *testing.T) {
 			wantQuery: "INSERT INTO `users` (`age`) VALUES (?)",
 			wantArgs:  []interface{}{1},
 		},
+		{
+			input:     Insert("users").Columns("age").Values(1).Schema("mydb"),
+			wantQuery: "INSERT INTO `mydb`.`users` (`age`) VALUES (?)",
+			wantArgs:  []interface{}{1},
+		},
 		{
 			input:     Dialect(dialect.Postgres).Insert("users").Columns("age").Values(1),
 			wantQuery: `INSERT INTO "users" ("age") VALUES ($1)`,
 			wantArgs:  []interface{}{1},
 		},
+		{
+			input:     Dialect(dialect.Postgres).Insert("users").Columns("age").Values(1).Schema("mydb"),
+			wantQuery: `INSERT INTO "mydb"."users" ("age") VALUES ($1)`,
+			wantArgs:  []interface{}{1},
+		},
+		{
+			input:     Dialect(dialect.SQLite).Insert("users").Columns("age").Values(1).Schema("mydb"),
+			wantQuery: "INSERT INTO `users` (`age`) VALUES (?)",
+			wantArgs:  []interface{}{1},
+		},
 		{
 			input:     Dialect(dialect.Postgres).Insert("users").Columns("age").Values(1).Returning("id"),
 			wantQuery: `INSERT INTO "users" ("age") VALUES ($1) RETURNING "id"`,
@@ -269,11 +287,26 @@ func TestBuilder(t *testing.T) {
 			wantQuery: "UPDATE `users` SET `name` = ?",
 			wantArgs:  []interface{}{"foo"},
 		},
+		{
+			input:     Update("users").Set("name", "foo").Schema("mydb"),
+			wantQuery: "UPDATE `mydb`.`users` SET `name` = ?",
+			wantArgs:  []interface{}{"foo"},
+		},
 		{
 			input:     Dialect(dialect.Postgres).Update("users").Set("name", "foo"),
 			wantQuery: `UPDATE "users" SET "name" = $1`,
 			wantArgs:  []interface{}{"foo"},
 		},
+		{
+			input:     Dialect(dialect.Postgres).Update("users").Set("name", "foo").Schema("mydb"),
+			wantQuery: `UPDATE "mydb"."users" SET "name" = $1`,
+			wantArgs:  []interface{}{"foo"},
+		},
+		{
+			input:     Dialect(dialect.SQLite).Update("users").Set("name", "foo").Schema("mydb"),
+			wantQuery: "UPDATE `users` SET `name` = ?",
+			wantArgs:  []interface{}{"foo"},
+		},
 		{
 			input:     Update("users").Set("name", "foo").Set("age", 10),
 			wantQuery: "UPDATE `users` SET `name` = ?, `age` = ?",
@@ -294,6 +327,20 @@ func TestBuilder(t *testing.T) {
 			wantQuery: `UPDATE "users" SET "name" = $1 WHERE "name" = $2`,
 			wantArgs:  []interface{}{"foo", "bar"},
 		},
+		{
+			input: func() Querier {
+				p1, p2 := EQ("name", "bar"), Or(EQ("age", 10), EQ("age", 20))
+				return Dialect(dialect.Postgres).
+					Update("users").
+					Set("name", "foo").
+					Where(p1).
+					Where(p2).
+					Where(p1).
+					Where(p2)
+			}(),
+			wantQuery: `UPDATE "users" SET "name" = $1 WHERE (("name" = $2 AND ("age" = $3 OR "age" = $4)) AND "name" = $5) AND ("age" = $6 OR "age" = $7)`,
+			wantArgs:  []interface{}{"foo", "bar", 10, 20, "bar", 10, 20},
+		},
 		{
 			input:     Update("users").Set("name", "foo").SetNull("spouse_id"),
 			wantQuery: "UPDATE `users` SET `spouse_id` = NULL, `name` = ?",
@@ -472,12 +519,32 @@ func TestBuilder(t *testing.T) {
 				Where(NotNull("parent_id")),
 			wantQuery: "DELETE FROM `users` WHERE `parent_id` IS NOT NULL",
 		},
+		{
+			input: Delete("users").
+				Where(NotNull("parent_id")).
+				Schema("mydb"),
+			wantQuery: "DELETE FROM `mydb`.`users` WHERE `parent_id` IS NOT NULL",
+		},
+		{
+			input: Dialect(dialect.SQLite).
+				Delete("users").
+				Where(NotNull("parent_id")).
+				Schema("mydb"),
+			wantQuery: "DELETE FROM `users` WHERE `parent_id` IS NOT NULL",
+		},
 		{
 			input: Dialect(dialect.Postgres).
 				Delete("users").
 				Where(IsNull("parent_id")),
 			wantQuery: `DELETE FROM "users" WHERE "parent_id" IS NULL`,
 		},
+		{
+			input: Dialect(dialect.Postgres).
+				Delete("users").
+				Where(IsNull("parent_id")).
+				Schema("mydb"),
+			wantQuery: `DELETE FROM "mydb"."users" WHERE "parent_id" IS NULL`,
+		},
 		{
 			input: Delete("users").
 				Where(And(IsNull("parent_id"), NotIn("name", "foo", "bar"))),
@@ -1284,6 +1351,21 @@ WHERE
 			wantQuery: "SELECT * FROM `s1`.`users` JOIN `s2`.`pets` AS `t0` ON `s1`.`users`.`id` = `t0`.`owner_id` WHERE `t0`.`name` = ?",
 			wantArgs:  []interface{}{"pedro"},
 		},
+		{
+			input: func() Querier {
+				t1, t2 := Table("users").Schema("s1"), Table("pets").Schema("s2")
+				sel := Select("*").
+					From(t1).Join(t2).
+					OnP(P(func(b *Builder) {
+						b.Ident(t1.C("id")).WriteOp(OpEQ).Ident(t2.C("owner_id"))
+					})).
+					Where(EQ(t2.C("name"), "pedro"))
+				sel.SetDialect(dialect.SQLite)
+				return sel
+			}(),
+			wantQuery: "SELECT * FROM `users` JOIN `pets` AS `t0` ON `users`.`id` = `t0`.`owner_id` WHERE `t0`.`name` = ?",
+			wantArgs:  []interface{}{"pedro"},
+		},
 	}
 	for i, tt := range tests {
 		t.Run(strconv.Itoa(i), func(t *testing.T) {
