@@ -102,7 +102,7 @@ func (d *MySQL) table(ctx context.Context, tx dialect.Tx, name string) (*Table,
 	for _, idx := range indexes {
 		t.AddIndex(idx.Name, idx.Unique, idx.columns)
 	}
-	if d.mariadb() {
+	if _, ok := d.mariadb(); ok {
 		if err := d.normalizeJSON(ctx, tx, t); err != nil {
 			return nil, err
 		}
@@ -281,6 +281,15 @@ func (d *MySQL) addColumn(c *Column) *sql.ColumnBuilder {
 	}
 	c.nullable(b)
 	c.defaultValue(b)
+	if c.Type == field.TypeJSON {
+		// Manually add a `CHECK` clause for older versions of MariaDB for validating the
+		// JSON documents. This constraint is automatically included from version 10.4.3.
+		if version, ok := d.mariadb(); ok && compareVersions(version, "10.4.3") == -1 {
+			b.Check(func(b *sql.Builder) {
+				b.WriteString("JSON_VALID(").Ident(c.Name).WriteByte(')')
+			})
+		}
+	}
 	return b
 }
 
@@ -405,7 +414,7 @@ func (d *MySQL) scanColumn(c *Column, rows *sql.Rows) error {
 	case "longblob":
 		c.Size = math.MaxUint32
 		c.Type = field.TypeBytes
-	case "varbinary":
+	case "binary", "varbinary":
 		c.Type = field.TypeBytes
 		c.Size = size
 	case "varchar":
@@ -571,9 +580,13 @@ func (d *MySQL) normalizeJSON(ctx context.Context, tx dialect.Tx, t *Table) erro
 	return rows.Close()
 }
 
-// mariadb reports if the migration runs on MariaDB.
-func (d *MySQL) mariadb() bool {
-	return strings.Contains(d.version, "MariaDB")
+// mariadb reports if the migration runs on MariaDB and returns the semver string.
+func (d *MySQL) mariadb() (string, bool) {
+	idx := strings.Index(d.version, "MariaDB")
+	if idx == -1 {
+		return "", false
+	}
+	return d.version[:idx-1], true
 }
 
 // parseColumn returns column parts, size and signed-info from a MySQL type.
@@ -591,7 +604,7 @@ func parseColumn(typ string) (parts []string, size int64, unsigned bool, err err
 		case len(parts) == 2: // int(10)
 			size, err = strconv.ParseInt(parts[1], 10, 0)
 		}
-	case "varbinary", "varchar", "char":
+	case "varbinary", "varchar", "char", "binary":
 		size, err = strconv.ParseInt(parts[1], 10, 64)
 	}
 	if err != nil {
