@@ -172,7 +172,10 @@ func NewType(c *Config, schema *load.Schema) (*Type, error) {
 	typ := &Type{
 		Config: c,
 		ID: &Field{
-			Name:      "id",
+			Name: "id",
+			def: &load.Field{
+				Name: "id",
+			},
 			Type:      idType,
 			StructTag: structTag("id", ""),
 		},
@@ -183,7 +186,7 @@ func NewType(c *Config, schema *load.Schema) (*Type, error) {
 		fields:      make(map[string]*Field, len(schema.Fields)),
 		foreignKeys: make(map[string]struct{}),
 	}
-	if err := typ.check(); err != nil {
+	if err := ValidSchemaName(typ.Name); err != nil {
 		return nil, err
 	}
 	for _, f := range schema.Fields {
@@ -483,9 +486,15 @@ func (t *Type) AddIndex(idx *load.Index) error {
 		return fmt.Errorf("missing fields or edges")
 	}
 	for _, name := range idx.Fields {
-		f, ok := t.fields[name]
-		if !ok {
-			return fmt.Errorf("unknown index field %q", name)
+		var f *Field
+		if name == t.ID.Name {
+			f = t.ID
+		} else {
+			var ok bool
+			f, ok = t.fields[name]
+			if !ok {
+				return fmt.Errorf("unknown index field %q", name)
+			}
 		}
 		if f.def.Size != nil && *f.def.Size > schema.DefaultStringLen {
 			return fmt.Errorf("field %q exceeds the index size limit (%d)", name, schema.DefaultStringLen)
@@ -671,17 +680,19 @@ func (t Type) RelatedTypes() []*Type {
 	return related
 }
 
-// check checks the schema type.
-func (t *Type) check() error {
-	pkg := t.Package()
+// ValidSchemaName will determine if a name is going to conflict with any
+// pre-defined names
+func ValidSchemaName(name string) error {
+	// schema package is lower-cased (see Type.Package)
+	pkg := strings.ToLower(name)
 	if token.Lookup(pkg).IsKeyword() {
 		return fmt.Errorf("schema lowercase name conflicts with Go keyword %q", pkg)
 	}
 	if types.Universe.Lookup(pkg) != nil {
 		return fmt.Errorf("schema lowercase name conflicts with Go predeclared identifier %q", pkg)
 	}
-	if _, ok := globalIdent[t.Name]; ok {
-		return fmt.Errorf("schema name conflicts with ent predeclared identifier %q", t.Name)
+	if _, ok := globalIdent[name]; ok {
+		return fmt.Errorf("schema name conflicts with ent predeclared identifier %q", name)
 	}
 	return nil
 }
@@ -726,6 +737,9 @@ func (f Field) UpdateDefaultName() string { return "Update" + f.DefaultName() }
 // DefaultValue returns the default value of the field. Invoked by the template.
 func (f Field) DefaultValue() interface{} { return f.def.DefaultValue }
 
+// DefaultFunc returns a bool stating if the default value is a func. Invoked by the template.
+func (f Field) DefaultFunc() interface{} { return f.def.DefaultKind == reflect.Func }
+
 // BuilderField returns the struct member of the field in the builder.
 func (f Field) BuilderField() string {
 	return builderField(f.Name)
@@ -813,6 +827,17 @@ func (f Field) MutationReset() string {
 	return name
 }
 
+// MutationSet returns the method name for setting the field value.
+// The default name is "Set<FieldName>". If the the method conflicts
+// with the mutation methods, suffix the method with "Field".
+func (f Field) MutationSet() string {
+	name := "Set" + f.StructField()
+	if _, ok := mutMethods[name]; ok {
+		name += "Field"
+	}
+	return name
+}
+
 // IsBool returns true if the field is a bool field.
 func (f Field) IsBool() bool { return f.Type != nil && f.Type.Type == field.TypeBool }
 
@@ -883,7 +908,8 @@ func (f Field) NullTypeField(rec string) string {
 	return expr
 }
 
-// Column returns the table column. It sets it as a primary key (auto_increment) in case of ID field.
+// Column returns the table column. It sets it as a primary key (auto_increment) in case of ID field, unless stated
+// otherwise.
 func (f Field) Column() *schema.Column {
 	c := &schema.Column{
 		Name:     f.StorageKey(),
@@ -907,6 +933,15 @@ func (f Field) Column() *schema.Column {
 	return c
 }
 
+// incremental returns if the column has an incremental behavior.
+// If no value is defined externally, we use a provided def flag
+func (f Field) incremental(def bool) bool {
+	if ant := f.EntSQL(); ant != nil && ant.Incremental != nil {
+		return *ant.Incremental
+	}
+	return def
+}
+
 // size returns the the field size defined in the schema.
 func (f Field) size() int64 {
 	if ant := f.EntSQL(); ant != nil && ant.Size != 0 {
@@ -924,7 +959,7 @@ func (f Field) PK() *schema.Column {
 		Name:      f.StorageKey(),
 		Type:      f.Type.Type,
 		Key:       schema.PrimaryKey,
-		Increment: true,
+		Increment: f.incremental(true),
 	}
 	// If the PK was defined by the user and it's UUID or string.
 	if f.UserDefined && !f.Type.Numeric() {
