@@ -142,6 +142,9 @@ func Sanity(t *testing.T, client *ent.Client) {
 	require := require.New(t)
 	ctx := context.Background()
 	usr := client.User.Create().SetName("foo").SetAge(20).SaveX(ctx)
+	client.User.Update().ExecX(ctx)
+	client.User.UpdateOne(usr).ExecX(ctx)
+	client.Node.Update().Where(node.ID(usr.ID)).ExecX(ctx)
 	require.Equal("foo", usr.Name)
 	require.Equal(20, usr.Age)
 	require.NotEmpty(usr.ID)
@@ -292,7 +295,7 @@ func Select(t *testing.T, client *ent.Client) {
 		Select(user.FieldName).
 		StringX(ctx)
 	require.Equal("foo", name)
-	client.User.Create().SetName("bar").SetAge(30).SaveX(ctx)
+	client.User.Create().SetName("bar").SetAge(30).AddFriends(u).SaveX(ctx)
 	t.Log("select one field with ordering")
 	names := client.User.
 		Query().
@@ -326,6 +329,25 @@ func Select(t *testing.T, client *ent.Client) {
 		ScanX(ctx, &v)
 	require.Equal([]int{30, 30, 30}, []int{v[0].Age, v[1].Age, v[2].Age})
 	require.Equal([]string{"bar", "baz", "foo"}, []string{v[0].Name, v[1].Name, v[2].Name})
+
+	users := client.User.
+		Query().
+		Select(user.FieldAge).
+		Where(user.Name("foo")).
+		WithFriends(func(q *ent.UserQuery) {
+			q.Select(user.FieldName)
+		}).
+		AllX(ctx)
+	for i := range users {
+		require.Empty(users[i].Name)
+		require.NotZero(users[i].ID)
+		require.NotZero(users[i].Age)
+		for _, f := range users[i].Edges.Friends {
+			require.NotEmpty(f.Name)
+			require.NotZero(f.ID)
+			require.Zero(f.Age)
+		}
+	}
 }
 
 func Predicate(t *testing.T, client *ent.Client) {
@@ -616,7 +638,7 @@ func Relation(t *testing.T, client *ent.Client) {
 	_, err = client.Group.UpdateOne(grp).SetMaxUsers(-10).Save(ctx)
 	require.Error(err, "max_users validator failed")
 	_, err = client.Group.Query().Select("unknown_field").String(ctx)
-	require.EqualError(err, "invalid field \"unknown_field\" for selection")
+	require.EqualError(err, "ent: invalid field \"unknown_field\" for query")
 	_, err = client.Group.Query().GroupBy("unknown_field").String(ctx)
 	require.EqualError(err, "invalid field \"unknown_field\" for group-by")
 	_, err = client.User.Query().Order(ent.Asc("invalid")).Only(ctx)
