@@ -28,6 +28,8 @@ func (User) Fields() []ent.Field {
 	return []ent.Field{
 		field.String("name").
 			Default("Unknown"),
+		field.Strings("foods").
+			Optional(),
 	}
 }
 
