@@ -12,7 +12,6 @@ import (
 	"math"
 	"reflect"
 	"regexp"
-	"sort"
 	"time"
 
 	"github.com/facebook/ent/schema"
@@ -208,6 +207,17 @@ func (b *stringBuilder) Default(s string) *stringBuilder {
 	return b
 }
 
+// DefaultFunc sets the function that is applied to set the default value
+// of the field on creation. For example:
+//
+//	field.String("cuid").
+//		DefaultFunc(cuid.New)
+//
+func (b *stringBuilder) DefaultFunc(fn interface{}) *stringBuilder {
+	b.desc.Default = fn
+	return b
+}
+
 // Nillable indicates that this field is a nillable.
 // Unlike "Optional" only fields, "Nillable" fields are pointers in the generated field.
 func (b *stringBuilder) Nillable() *stringBuilder {
@@ -285,6 +295,9 @@ func (b *stringBuilder) Annotations(annotations ...schema.Annotation) *stringBui
 
 // Descriptor implements the ent.Field interface by returning its descriptor.
 func (b *stringBuilder) Descriptor() *Descriptor {
+	if b.desc.Default != nil {
+		b.desc.checkDefaultFunc(stringType)
+	}
 	return b.desc
 }
 
@@ -484,6 +497,17 @@ func (b *bytesBuilder) Default(v []byte) *bytesBuilder {
 	return b
 }
 
+// DefaultFunc sets the function that is applied to set the default value
+// of the field on creation. For example:
+//
+//	field.Bytes("cuid").
+//		DefaultFunc(cuid.New)
+//
+func (b *bytesBuilder) DefaultFunc(fn interface{}) *bytesBuilder {
+	b.desc.Default = fn
+	return b
+}
+
 // Nillable indicates that this field is a nillable.
 // Unlike "Optional" only fields, "Nillable" fields are pointers in the generated field.
 func (b *bytesBuilder) Nillable() *bytesBuilder {
@@ -569,6 +593,9 @@ func (b *bytesBuilder) SchemaType(types map[string]string) *bytesBuilder {
 
 // Descriptor implements the ent.Field interface by returning its descriptor.
 func (b *bytesBuilder) Descriptor() *Descriptor {
+	if b.desc.Default != nil {
+		b.desc.checkDefaultFunc(bytesType)
+	}
 	return b.desc
 }
 
@@ -672,7 +699,7 @@ func (b *enumBuilder) Values(values ...string) *enumBuilder {
 //
 func (b *enumBuilder) NamedValues(namevalue ...string) *enumBuilder {
 	if len(namevalue)%2 == 1 {
-		b.desc.err = fmt.Errorf("Enum.NamedValues: odd argument count")
+		b.desc.Err = fmt.Errorf("Enum.NamedValues: odd argument count")
 		return b
 	}
 	for i := 0; i < len(namevalue); i += 2 {
@@ -681,26 +708,6 @@ func (b *enumBuilder) NamedValues(namevalue ...string) *enumBuilder {
 	return b
 }
 
-// ValueMap adds the given values in the map to the enum value.
-// The key in the map is the Go identifier and the value in the
-// map is the actual enum value.
-//
-// If keys in not titled, ent codegen will change it to be exported.
-//
-// Deprecated: the ValueMap method predates the NamedValues method and it
-// is planned be removed in v0.5.0. New code should use NamedValues instead.
-func (b *enumBuilder) ValueMap(values map[string]string) *enumBuilder {
-	enums := make([]struct{ N, V string }, 0, len(values))
-	for k, v := range values {
-		enums = append(enums, struct{ N, V string }{N: k, V: v})
-	}
-	sort.Slice(enums, func(i, j int) bool {
-		return enums[i].V < enums[j].V
-	})
-	b.desc.Enums = append(b.desc.Enums, enums...)
-	return b
-}
-
 // Default sets the default value of the field.
 func (b *enumBuilder) Default(value string) *enumBuilder {
 	b.desc.Default = value
@@ -844,7 +851,7 @@ func (b *uuidBuilder) StructTag(s string) *uuidBuilder {
 func (b *uuidBuilder) Default(fn interface{}) *uuidBuilder {
 	typ := reflect.TypeOf(fn)
 	if typ.Kind() != reflect.Func || typ.NumIn() != 0 || typ.NumOut() != 1 || typ.Out(0).String() != b.desc.Info.String() {
-		b.desc.err = fmt.Errorf("expect type (func() %s) for uuid default value", b.desc.Info)
+		b.desc.Err = fmt.Errorf("expect type (func() %s) for uuid default value", b.desc.Info)
 	}
 	b.desc.Default = fn
 	return b
@@ -899,12 +906,7 @@ type Descriptor struct {
 	Sensitive     bool                    // sensitive info string field.
 	SchemaType    map[string]string       // override the schema type.
 	Annotations   []schema.Annotation     // field annotations.
-	err           error
-}
-
-// Err returns the error, if any, that was added by the field builder.
-func (d *Descriptor) Err() error {
-	return d.err
+	Err           error
 }
 
 func (d *Descriptor) goType(typ interface{}, expectType reflect.Type) {
@@ -915,6 +917,7 @@ func (d *Descriptor) goType(typ interface{}, expectType reflect.Type) {
 		Ident:   tv.String(),
 		PkgPath: tv.PkgPath(),
 		RType: &RType{
+			rtype:   tv,
 			Name:    tv.Name(),
 			Kind:    tv.Kind(),
 			PkgPath: tv.PkgPath(),
@@ -944,14 +947,32 @@ func (d *Descriptor) goType(typ interface{}, expectType reflect.Type) {
 			info.RType.Methods[m.Name] = struct{ In, Out []*RType }{in, out}
 		}
 	default:
-		d.err = fmt.Errorf("GoType must be a %q type or ValueScanner", expectType)
+		d.Err = fmt.Errorf("GoType must be a %q type or ValueScanner", expectType)
 		if pt := reflect.PtrTo(t); pt.Implements(valueScannerType) {
-			d.err = fmt.Errorf("%s. Use %s instead", d.err, pt)
+			d.Err = fmt.Errorf("%s. Use %s instead", d.Err, pt)
 		}
 	}
 	d.Info = info
 }
 
+func (d *Descriptor) checkDefaultFunc(expectType reflect.Type) {
+	typ := reflect.TypeOf(d.Default)
+	if typ.Kind() != reflect.Func || d.Err != nil {
+		return
+	}
+	err := fmt.Errorf("expect type (func() %s) for default value", d.Info)
+	if typ.NumIn() != 0 || typ.NumOut() != 1 {
+		d.Err = err
+	}
+	rtype := expectType
+	if d.Info.RType != nil {
+		rtype = d.Info.RType.rtype
+	}
+	if !typ.Out(0).AssignableTo(rtype) {
+		d.Err = err
+	}
+}
+
 var (
 	boolType         = reflect.TypeOf(false)
 	bytesType        = reflect.TypeOf([]byte(nil))
